#include <X11/Intrinsic.h>

#include "wms.h"
#include "wms_core.h"
#include "wmsstats.h"

Boolean _wms_tracing = True;
 
static short indent_level = 0;
static void indent ();

void _Xwms_class_initialize (wc)
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original initialize method */
      if (extension->class_initialize != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("class_initialize(%s): entered\n",
               wc->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->class_initialize) ();
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsClassStatEntry (extension, wc, (WidgetClass) NULL, "class_initialize",
            &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_class_part_initialize (wc, realwc)
   WidgetClass wc;
   WidgetClass realwc;
{
   register WidgetStatisticsExtension extension;
   CoreMethods cm;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (realwc);

   cm = _XwmsFindWidgetClass (realwc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original class_part_initialize method */
      if (extension->class_part_initialize != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("class_part_initialize: (%s) part of %s: entered\n",
               realwc->core_class.class_name, wc->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->class_part_initialize) (wc);
         _Xwms_time (&end);
 
         /* store the statistics */
         _XwmsClassStatEntry (extension, realwc, wc, "class_part_initialize", 
            &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }

      /* at this point binding of the Xwms methods can occur because all
         core class XtInherit... methods will have been replaced */
      if (wc == realwc)
      {
#ifdef DEBUG
   printf ("_XwmsClassPartInitialize: Should only be here once per class (%s)\n", 
      realwc->core_class.class_name);
#endif

         if (extension->realize == _XwmsInheritMethodLater)
            _XwmsInstallRealize (cm, wc, extension);

         if (extension->resize == _XwmsInheritMethodLater)
            _XwmsInstallResize (cm, wc, extension);

         if (extension->expose == _XwmsInheritMethodLater)
            _XwmsInstallExpose (cm, wc, extension);

         if (extension->set_values_almost == _XwmsInheritMethodLater)
            _XwmsInstallSetValuesAlmost (cm, wc, extension);

         if (extension->accept_focus == _XwmsInheritMethodLater)
            _XwmsInstallAcceptFocus (cm, wc, extension);

         if (extension->query_geometry == _XwmsInheritMethodLater)
            _XwmsInstallQueryGeometry (cm, wc, extension);

         if (extension->display_accelerator == _XwmsInheritMethodLater)
            _XwmsInstallDisplayAccelerator (cm, wc, extension);
      }
   }
}

void _Xwms_initialize (request, new, args, num_args, wc)
   Widget request;
   Widget new;
   ArgList args;
   Cardinal *num_args;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      if (extension->initialize != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("initialize.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (request),
               (XtClass (request))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->initialize) (request, new, args, num_args);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, request, "initialize", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_initialize_hook (w, args, num_args, wc)
   Widget w;
   ArgList args;
   Cardinal *num_args;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original initialize_hook method */
      if (extension->initialize_hook != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("initialize_hook.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->initialize_hook) (w, args, num_args);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "initialize_hook", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_realize (w, mask, attr, wc)
   Widget w;
   XtValueMask *mask;
   XSetWindowAttributes *attr;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original realize method */
      if (extension->realize != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("realize.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->realize) (w, mask, attr);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "realize", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_destroy (w, wc)
   Widget w;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original destroy method */
      if (extension->destroy != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("destroy.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->destroy) (w);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "destroy", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_resize (w, wc)
   Widget w;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original resize method */
      if (extension->resize != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("resize.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->resize) (w);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "resize", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_expose (w, event, region, wc)
   Widget w;
   XEvent *event;
   Region region;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original expose method */
      if (extension->expose != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("expose.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->expose) (w, event, region);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "expose", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

Boolean _Xwms_set_values (old, request, new, args, num_args, wc)
   Widget old;
   Widget request;
   Widget new;
   ArgList args;
   Cardinal *num_args;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;
   Boolean do_redisplay = False;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original set_values method */
      if (extension->set_values != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("set_values.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (old),
               (XtClass (old))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         do_redisplay = (extension->set_values) (old, request, new, args, 
            num_args);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, old, "set_values", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }

   /* return the value returned from the widget's set_values method */
   return (do_redisplay);
}

Boolean _Xwms_set_values_hook (w, args, num_args, wc)
   Widget w;
   ArgList args;
   Cardinal *num_args;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;
   Boolean do_redisplay = False;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original set_values_hook method */
      if (extension->set_values_hook != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("set_values_hook.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         do_redisplay = (extension->set_values_hook) (w, args, num_args);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "set_values_hook", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }

   /* return the value returned from the widget's set_values_hook method */
   return (do_redisplay);
}

void _Xwms_set_values_almost (w, new_widget_return, request, reply, wc)
   Widget w;
   Widget new_widget_return;
   XtWidgetGeometry *request;
   XtWidgetGeometry *reply;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original set_values_almost method */
      if (extension->set_values_almost != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("set_values_almost.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->set_values_almost) (w, new_widget_return, request, reply);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "set_values_almost", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

void _Xwms_get_values_hook (w, args, num_args, wc)
   Widget w;
   ArgList args;
   Cardinal *num_args;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original get_values_hook method */
      if (extension->get_values_hook != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("get_values_hook.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->get_values_hook) (w, args, num_args);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "get_values_hook", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}

Boolean _Xwms_accept_focus (w, time, wc)
   Widget w;
   Time *time;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;
   Boolean accept_focus = False;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original accept_focus method */
      if (extension->accept_focus != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("accept_focus.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         accept_focus = (extension->accept_focus) (w, time);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "accept_focus", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }

   /* return the value returned from the widget's accept_focus method */
   return (accept_focus);
}

XtGeometryResult _Xwms_query_geometry (w, request, geometry_return, wc)
   Widget w;
   XtWidgetGeometry *request;
   XtWidgetGeometry *geometry_return;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;
   XtGeometryResult geometry_result = XtGeometryYes;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original query_geometry method */
      if (extension->query_geometry != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("query_geometry.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         geometry_result = (extension->query_geometry) (w, request, 
            geometry_return);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "query_geometry", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }

   /* return the value returned from the widget's query_geometry method */
   return (geometry_result);
}

void _Xwms_display_accelerator (w, string, wc)
   Widget w;
   String string;
   WidgetClass wc;
{
   register WidgetStatisticsExtension extension;

   /* find the WidgetStatisticsExtension */
   extension = _XwmsFindExtension (wc);

   if (extension != (WidgetStatisticsExtension) NULL)
   {
      /* call the widget's original display_accelerator method */
      if (extension->display_accelerator != NULL)
      {
         struct timestruc_t begin, end;

         if (_wms_tracing)
         {
            indent (indent_level);
            printf ("display_accelerator.%s for widget %s.%s: entering\n", 
               wc->core_class.class_name, XtName (w),
               (XtClass (w))->core_class.class_name);
            indent_level++;
         }

         _Xwms_time (&begin);
         (extension->display_accelerator) (w, string);
         _Xwms_time (&end);

         /* store the statistics */
         _XwmsStatEntry (extension, wc, w, "display_accelerator", &begin, &end);

         if (_wms_tracing)
         {
            indent_level--;
         }
      }
   }
}





void indent (indent_level)
   short indent_level;
{
   while (indent_level-- > 0)
      printf ("   ");
}
