#include <X11/Intrinsic.h>

#include "wms.h"
#include "wms_core.h"

void XwmsSelectMethod (widget_class, method_mask)
   WidgetClass widget_class;
   long method_mask;
{
   CoreMethods cm;
   WidgetStatisticsExtension extension;
   Boolean need_class_part_initialize = False;
   static Boolean _Xwms_initialized = False;

   /* initialize the Xwms package if it hasn't been already */
   if (_Xwms_initialized == False)
   {
      _XwmsInitialize ();
      _Xwms_initialized = True;
   }

   /* If there is no mask, then there is nothing to do */
   if (method_mask == NoMethodMask)
      return;

   /* Find the record containing pointers to the one line methods for
      the widget class passed in.  If it can't be found then we cannot
      monitor it. */
   if ((cm = _XwmsFindWidgetClass (widget_class)) == (CoreMethods) NULL)
      return;

   /* Find the extension - _XwmsInstallExtension will install it if it isn't
      already */
   extension = _XwmsInstallExtension (widget_class);

   if (method_mask & ClassInitializeMask)
      _XwmsInstallClassInitialize (cm, widget_class, extension);

   if (method_mask & ClassPartInitializeMask)
      _XwmsInstallClassPartInitialize (cm, widget_class, extension);

   if (method_mask & InitializeMask)
      _XwmsInstallInitialize (cm, widget_class, extension);

   if (method_mask & InitializeHookMask)
      _XwmsInstallInitializeHook (cm, widget_class, extension);

   if (method_mask & RealizeMask)
   {
      extension->realize = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & DestroyMask)
      _XwmsInstallDestroy (cm, widget_class, extension);

   if (method_mask & ResizeMask)
   {
      extension->resize = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & ExposeMask)
   {
      extension->expose = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & SetValuesMask)
      _XwmsInstallSetValues (cm, widget_class, extension);

   if (method_mask & SetValuesHookMask)
      _XwmsInstallSetValuesHook (cm, widget_class, extension);

   if (method_mask & SetValuesAlmostMask)
   {
      extension->set_values_almost = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & GetValuesHookMask)
      _XwmsInstallGetValuesHook (cm, widget_class, extension);

   if (method_mask & AcceptFocusMask)
   {
      extension->accept_focus = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & QueryGeometryMask)
   {
      extension->query_geometry = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   if (method_mask & DisplayAcceleratorMask)
   {
      extension->display_accelerator = _XwmsInheritMethodLater;
      need_class_part_initialize = True;
   }

   /* if we haven't hooked into the class_part_initialize method for
      class, and we need to, then do it now.  NULL at this point means
      that there is no class_part_initialize method to trace (whether or
      not the user requested it to be traced.  The flag need_class_part_initialize
      means we need to add our class_part_initialize_method to delay method swapping
      until the class is initialized. */
   if (extension->class_part_initialize == NULL && 
         need_class_part_initialize == True)
      _XwmsSpecialInstallClassPartInitialize (cm, widget_class, extension);
      
}
