#include <X11/Intrinsic.h>

#include "wms.h"

/* WMS global variables for file collection of statistics */
FILE *wms_fp = (FILE *) NULL;
char *wms_stat_file_name = (char *) NULL;
 
WidgetStatisticsExtension _XwmsFindExtension (widget_class)
   WidgetClass widget_class;
{
   static XrmQuark widgetStatisticsExtension = NULLQUARK;
   WidgetStatisticsExtension extension;

   if (widgetStatisticsExtension == NULLQUARK)
      widgetStatisticsExtension = 
         XrmStringToQuark ("WidgetStatisticsExtension");

   /* search for the WidgetStatisticsExtension */
   for (extension = (WidgetStatisticsExtension) widget_class->core_class.extension;
      extension != NULL && extension->record_type != widgetStatisticsExtension;
      extension = (WidgetStatisticsExtension) extension->next_extension)
   {
      /* intentionally empty */
   }

   return (extension);
}


WidgetStatisticsExtension _XwmsInstallExtension (widget_class)
   WidgetClass widget_class;
{
   static XrmQuark widgetStatisticsExtension = NULLQUARK;
   WidgetStatisticsExtension extension;

   extension = _XwmsFindExtension (widget_class);

   /* if the WidgetStatisticsExtension was not found, add it */
   if (extension == NULL)
   {
      if (widgetStatisticsExtension == NULLQUARK)
         widgetStatisticsExtension = 
            XrmStringToQuark ("WidgetStatisticsExtension");

      extension = XtNew (WidgetStatisticsExtensionRec);
      extension->next_extension = widget_class->core_class.extension;
      widget_class->core_class.extension = (XtPointer) extension;
      extension->record_type = widgetStatisticsExtension;
      extension->version = XWMS_VERSION;
      extension->record_size = sizeof (WidgetStatisticsExtensionRec);

      extension->trace_class_part_initialize = False;
      extension->class_part_initialize = NULL;

      extension->initialize = NULL;
      extension->initialize_hook = NULL;
      extension->realize = NULL;
      extension->destroy = NULL;
      extension->resize = NULL;
      extension->expose = NULL;
      extension->set_values = NULL;
      extension->set_values_hook = NULL;
      extension->set_values_almost = NULL;
      extension->get_values_hook = NULL;
      extension->accept_focus = NULL;
      extension->query_geometry = NULL;
      extension->display_accelerator = NULL;

      /* The same file is used to record all data.  Only open it if it hasn't
         been already.  It should be opened the first time the extension is
         installed for a widget class. */
      if (wms_fp == (FILE *) NULL)
      {
         /* If the user didn't change the file name using XwmsStatFile ()
            the global variable wms_stat_file_name will be NULL.  We must
            set it before we can open the data file. */
         if (wms_stat_file_name == (char *) NULL)
            wms_stat_file_name = strdup ("wms.data");
         if ((wms_fp = fopen (wms_stat_file_name, "w")) == NULL)
         {
            printf ("opening file %s\n failed", wms_stat_file_name);
         }
      }
   }

   /* return the extension record */
   return (extension);
}
